@@ -1,5 +1,9 @@
+#!/usr/bin/env python
+
 # Copyright (C) 2006 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -24,14 +28,13 @@ from __future__ import print_function
 import logging
 
 try:
-    from gi.repository import GObject as gobject
+    from gi.repository import GLib
 except ImportError:
     raise SystemExit(77)
 
 import dbus.glib
 from dbus import SessionBus
 from dbus.service import BusName
-from dbus._compat import is_py2
 
 from crosstest import (
     CROSS_TEST_BUS_NAME, CROSS_TEST_PATH, INTERFACE_CALLBACK_TESTS,
@@ -222,12 +225,7 @@ class TestsImpl(dbus.service.Object):
         logger.info('Sum of %r is %r', input, x)
         return x
 
-
-    kwargs = {}
-    if is_py2:
-        kwargs['utf8_strings'] = True
-
-    @dbus.service.method(INTERFACE_TESTS, 'a{ss}', 'a{sas}', **kwargs)
+    @dbus.service.method(INTERFACE_TESTS, 'a{ss}', 'a{sas}')
     def InvertMapping(self, input):
         tested_things.add(INTERFACE_TESTS + '.InvertMapping')
         output = dbus.Dictionary({})
@@ -272,15 +270,17 @@ class TestsImpl(dbus.service.Object):
 
     @dbus.service.method(INTERFACE_TESTS, 'st', '',
                          connection_keyword='conn',
-                         **kwargs)
-    def Trigger(self, object, parameter, conn=None):
+                         async_callbacks=('reply_cb', 'error_cb'))
+    def Trigger(self, object, parameter, conn=None, reply_cb=None,
+                error_cb=None):
         assert isinstance(object, str)
         logger.info('method/signal: client wants me to emit Triggered(%r) from %r', parameter, object)
         tested_things.add(INTERFACE_TESTS + '.Trigger')
-        gobject.idle_add(lambda: self.emit_Triggered_from(conn, object,
-                                                          parameter))
-    
-    def emit_Triggered_from(self, conn, object, parameter):
+        GLib.idle_add(lambda: self.emit_Triggered_from(conn, object,
+                                                       parameter,
+                                                       reply_cb))
+
+    def emit_Triggered_from(self, conn, object, parameter, reply_cb):
         assert isinstance(object, str)
         logger.info('method/signal: Emitting Triggered(%r) from %r', parameter, object)
         obj = objects.get(object, None)
@@ -289,6 +289,8 @@ class TestsImpl(dbus.service.Object):
             objects[object] = obj
         obj.Triggered(parameter)
         logger.info('method/signal: Emitted Triggered')
+        reply_cb()
+        logger.info('method/signal: Sent reply for Tests.Trigger()')
 
     @dbus.service.method(INTERFACE_TESTS, '', '')
     def Exit(self):
@@ -298,7 +300,7 @@ class TestsImpl(dbus.service.Object):
             if x not in tested_things:
                 print('%s untested' % x)
         logger.info('will quit when idle')
-        gobject.idle_add(self._exit_fn)
+        GLib.idle_add(self._exit_fn)
 
 
 class Server(SingleTestsImpl, TestsImpl, SignalTestsImpl):
@@ -316,20 +318,16 @@ class Server(SingleTestsImpl, TestsImpl, SignalTestsImpl):
 if __name__ == '__main__':
     bus = SessionBus()
     bus_name = BusName(CROSS_TEST_BUS_NAME, bus=bus)
-    loop = gobject.MainLoop()
+    loop = GLib.MainLoop()
     obj = Server(bus_name, CROSS_TEST_PATH, loop.quit)
     objects[CROSS_TEST_PATH] = obj
-    kwargs = {}
-    if is_py2:
-        kwargs['utf8_strings'] = True
     bus.add_signal_receiver(obj.triggered_by_client,
                             signal_name='Trigger',
                             dbus_interface=INTERFACE_SIGNAL_TESTS,
                             named_service=None,
                             path=None,
                             sender_keyword='sender',
-                            path_keyword='sender_path',
-                            **kwargs)
+                            path_keyword='sender_path')
 
     logger.info("running...")
     loop.run()
