@@ -3,6 +3,8 @@
 # Copyright (C) 2004 Red Hat Inc. <http://www.redhat.com/>
 # Copyright (C) 2005-2007 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -29,28 +31,35 @@ import unittest
 import time
 import logging
 import weakref
-
-builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
-pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
+import sys
 
 import dbus
 import _dbus_bindings
 import dbus.glib
+import dbus.lowlevel
 import dbus.service
+import dbus_test_utils
 
-from dbus._compat import is_py2, is_py3
-from gi.repository import GObject as gobject
-
+try:
+    from gi.repository import GLib, Gio
+except ImportError:
+    print('1..0 # SKIP cannot import GLib')
+    raise SystemExit(0)
 
 logging.basicConfig()
 
+if 'DBUS_TEST_UNINSTALLED' in os.environ:
+    builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
+    pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
+    pkg = dbus.__file__
 
-pkg = dbus.__file__
-if not pkg.startswith(pydir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%pkg)
+    if not pkg.startswith(pydir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % pkg)
 
-if not _dbus_bindings.__file__.startswith(builddir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%_dbus_bindings.__file__)
+    if not _dbus_bindings.__file__.startswith(builddir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % _dbus_bindings.__file__)
 
 test_types_vals = [1, 12323231, 3.14159265, 99999999.99,
                  "dude", "123", "What is all the fuss about?", "gob@gob.com",
@@ -69,6 +78,11 @@ NAME = "org.freedesktop.DBus.TestSuitePythonService"
 IFACE = "org.freedesktop.DBus.TestSuiteInterface"
 OBJECT = "/org/freedesktop/DBus/TestSuitePythonObject"
 
+# A random string that we should not transmit on the bus as a result of
+# the NO_REPLY flag
+SHOULD_NOT_HAPPEN = u'a1c04a41-cf98-4923-8487-ddaeeb3f02d1'
+
+
 class TestDBusBindings(unittest.TestCase):
     def setUp(self):
         self.bus = dbus.SessionBus()
@@ -113,16 +127,9 @@ class TestDBusBindings(unittest.TestCase):
         self.assertEqual(self.iface.AcceptListOfByte(b'\1\2\3', byte_arrays=True), b'\1\2\3')
         self.assertEqual(self.iface.AcceptByteArray(b'\1\2\3'), [1,2,3])
         self.assertEqual(self.iface.AcceptByteArray(b'\1\2\3', byte_arrays=True), b'\1\2\3')
-        if is_py2:
-            self.assertTrue(isinstance(self.iface.AcceptUTF8String('abc'), unicode))
-            self.assertTrue(isinstance(self.iface.AcceptUTF8String('abc', utf8_strings=True), str))
-        unicode_type = (str if is_py3 else unicode)
         self.assertTrue(isinstance(self.iface.AcceptUnicodeString('abc'), 
-                                   unicode_type))
-        kwargs = {}
-        if is_py2:
-            kwargs['utf8_strings'] = True
-        self.assertTrue(isinstance(self.iface.AcceptUnicodeString('abc', **kwargs), str))
+                                   str))
+        self.assertTrue(isinstance(self.iface.AcceptUnicodeString('abc'), str))
 
     def testIntrospection(self):
         #test introspection
@@ -164,17 +171,6 @@ class TestDBusBindings(unittest.TestCase):
         self.assertTrue(destination.startswith(':'))
         self.assertEqual(message_cls, 'dbus.lowlevel.MethodCallMessage')
 
-    def testUtf8StringsSync(self):
-        if is_py3:
-            return
-        send_val = 'foo'
-        recv_val = self.iface.Echo(send_val, utf8_strings=True)
-        self.assertTrue(isinstance(recv_val, str))
-        self.assertTrue(isinstance(recv_val, dbus.UTF8String))
-        recv_val = self.iface.Echo(send_val, utf8_strings=False)
-        self.assertTrue(isinstance(recv_val, unicode))
-        self.assertTrue(isinstance(recv_val, dbus.String))
-
     def testBenchmarkIntrospect(self):
         print("\n********* Benchmark Introspect ************")
         a = time.time()
@@ -185,21 +181,75 @@ class TestDBusBindings(unittest.TestCase):
         print("Delta: %f" % (b - a))
         self.assertTrue(True)
 
+    def testAllowInteractiveAuthorization(self):
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestAllowInteractiveAuthorization')
+        assert not message.get_allow_interactive_authorization()
+        message.set_allow_interactive_authorization(True)
+        assert message.get_allow_interactive_authorization()
+
+    def testNoReply(self):
+        failures = []
+        report = []
+        main_loop = GLib.MainLoop()
+
+        def message_filter(conn, m):
+            print('Message filter received message: %r, %r' % (m, m.get_args_list()))
+
+            if conn is not self.bus:
+                failures.append('Message filter called on unexpected bus')
+
+            for a in m.get_args_list():
+                if isinstance(a, str):
+                    if SHOULD_NOT_HAPPEN in a:
+                        failures.append('Had an unexpected reply')
+                    elif a == 'TestNoReply report':
+                        report.extend(m.get_args_list())
+                        main_loop.quit()
+
+            return dbus.lowlevel.HANDLER_RESULT_NOT_YET_HANDLED
+
+        self.bus.add_message_filter(message_filter)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(True)
+        message.append(False)
+        message.set_no_reply(True)
+        self.bus.send_message(message)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(False)
+        message.append(False)
+        message.set_no_reply(True)
+        self.bus.send_message(message)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(True)
+        message.append(True)
+        self.bus.send_message(message)
+
+        main_loop.run()
+        self.bus.remove_message_filter(message_filter)
+
+        if failures:
+            self.assertTrue(False, failures)
+
+        self.assertEqual(report[0], 'TestNoReply report')
+        self.assertEqual(report[1], 1)
+        self.assertEqual(report[2], 1)
+
     def testAsyncCalls(self):
         #test sending python types and getting them back async
         print("\n********* Testing Async Calls ***********")
 
         failures = []
-        main_loop = gobject.MainLoop()
+        main_loop = GLib.MainLoop()
 
         class async_check:
             def __init__(self, test_controler, expected_result, do_exit, **kwargs):
                 self.expected_result = expected_result
                 self.do_exit = do_exit
                 self.test_controler = test_controler
-                if is_py2:
-                    self.utf8 = kwargs['utf8']
-                elif 'utf8' in kwargs:
+                if 'utf8' in kwargs:
                     raise TypeError("unexpected keyword argument 'utf8'")
 
             def callback(self, val):
@@ -209,13 +259,6 @@ class TestDBusBindings(unittest.TestCase):
 
                     self.test_controler.assertEqual(val, self.expected_result)
                     self.test_controler.assertEqual(val.variant_level, 1)
-                    if is_py2:
-                        if self.utf8 and not isinstance(val, dbus.UTF8String):
-                            failures.append('%r should have been utf8 but was not' % val)
-                            return
-                        elif not self.utf8 and isinstance(val, dbus.UTF8String):
-                            failures.append('%r should not have been utf8' % val)
-                            return
                 except Exception as e:
                     failures.append("%s:\n%s" % (e.__class__, e))
 
@@ -229,17 +272,10 @@ class TestDBusBindings(unittest.TestCase):
         last_type = test_types_vals[-1]
         for send_val in test_types_vals:
             print("Testing %s" % str(send_val))
-            kwargs = {}
-            if is_py2:
-                utf8 = (send_val == 'gob@gob.com')
-                kwargs['utf8'] = utf8
-                kwargs['utf8_strings'] = utf8
-            check = async_check(self, send_val, last_type == send_val,
-                                **kwargs)
+            check = async_check(self, send_val, last_type == send_val)
             recv_val = self.iface.Echo(send_val,
                                        reply_handler=check.callback,
-                                       error_handler=check.error_handler,
-                                       **kwargs)
+                                       error_handler=check.error_handler)
         main_loop.run()
         if failures:
             self.assertTrue(False, failures)
@@ -322,12 +358,12 @@ class TestDBusBindings(unittest.TestCase):
 
     def testAsyncMethods(self):
         print("\n********* Testing asynchronous method implementation *******")
-        for async in (True, False):
+        for async_ in (True, False):
             for fail in (True, False):
                 try:
                     val = ('a', 1, False, [1,2], {1:2})
-                    print("calling AsynchronousMethod with %s %s %s" % (async, fail, val))
-                    ret = self.iface.AsynchronousMethod(async, fail, val)
+                    print("calling AsynchronousMethod with %s %s %s" % (async_, fail, val))
+                    ret = self.iface.AsynchronousMethod(async_, fail, val)
                 except Exception as e:
                     self.assertTrue(fail, '%s: %s' % (e.__class__, e))
                     print("Expected failure: %s: %s" % (e.__class__, e))
@@ -508,7 +544,7 @@ class TestDBusBindings(unittest.TestCase):
         dbus.Bus(private=True).close()
 
     def testTimeoutAsyncClient(self):
-        loop = gobject.MainLoop()
+        loop = GLib.MainLoop()
         passes = []
         fails = []
         def correctly_returned():
@@ -563,6 +599,8 @@ class TestDBusBindings(unittest.TestCase):
             self.iface.RaiseDBusExceptionNoTraceback()
         except Exception as e:
             self.assertTrue(isinstance(e, dbus.DBusException), e.__class__)
+            self.assertEqual(e.get_dbus_name(),
+                    'com.example.Networking.ServerError')
             self.assertEqual(str(e),
                               'com.example.Networking.ServerError: '
                               'Server not responding')
@@ -573,12 +611,67 @@ class TestDBusBindings(unittest.TestCase):
             self.iface.RaiseDBusExceptionWithTraceback()
         except Exception as e:
             self.assertTrue(isinstance(e, dbus.DBusException), e.__class__)
+            self.assertEqual(e.get_dbus_name(),
+                    'com.example.Misc.RealityFailure')
             self.assertTrue(str(e).startswith('com.example.Misc.RealityFailure: '
                                            'Traceback '),
                          'Wanted a traceback but got:\n%s' % str(e))
         else:
             raise AssertionError('Wanted an exception')
 
+    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix")
+    def test_invalid_fd_index(self) -> None:
+        # libdbus cannot call a method containing fd indexes that are
+        # out-of-range for the number of fds attached to the message,
+        # but Gio can.
+        gdbus_conn = Gio.bus_get_sync(Gio.BusType.SESSION, None)
+
+        try:
+            results, fds = gdbus_conn.call_with_unix_fd_list_sync(
+                NAME, OBJECT, IFACE, 'EchoVariant',
+                GLib.Variant('(v)', (GLib.Variant.new_handle(42),)),
+                GLib.VariantType('(v)'),
+                Gio.DBusCallFlags.NONE,
+                -1,
+                Gio.UnixFDList.new(),
+                None,
+            )
+        except GLib.GError as e:
+            self.assertEqual(
+                Gio.dbus_error_get_remote_error(e),
+                'org.freedesktop.DBus.Python.ValueError',
+            )
+            self.assertIn(
+                'invalid file descriptor in message', str(e),
+            )
+        else:
+            raise AssertionError('Wanted an exception')
+
+        plain_fd = os.open('/dev/null', os.O_RDONLY)
+        expected_stat = os.fstat(plain_fd)
+
+        try:
+            # Check that the server can do a similar operation with valid
+            # fds, and has not crashed
+            for i in range(2):
+                fds_in = Gio.UnixFDList.new()
+                self.assertEqual(fds_in.append(plain_fd), 0)
+                results, fds_out = gdbus_conn.call_with_unix_fd_list_sync(
+                    NAME, OBJECT, IFACE, 'EchoVariant',
+                    GLib.Variant('(v)', (GLib.Variant.new_handle(0),)),
+                    GLib.VariantType('(v)'),
+                    Gio.DBusCallFlags.NONE,
+                    -1,
+                    fds_in,
+                    None,
+                )
+                self.assertEqual(results[0], 0)
+                fd = fds_out.peek_fds()[0]
+                actual_stat = os.fstat(fd)
+                self.assertEqual(expected_stat, actual_stat)
+        finally:
+            os.close(plain_fd)
+
 """ Remove this for now
 class TestDBusPythonToGLibBindings(unittest.TestCase):
     def setUp(self):
@@ -615,7 +708,6 @@ class TestDBusPythonToGLibBindings(unittest.TestCase):
             self.assertEqual(send_val, recv_val.object)
 """
 if __name__ == '__main__':
-    gobject.threads_init()
     dbus.glib.init_threads()
 
-    unittest.main()
+    dbus_test_utils.main()
