@@ -3,6 +3,8 @@
 # Copyright (C) 2004 Red Hat Inc. <http://www.redhat.com/>
 # Copyright (C) 2005-2007 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -27,17 +29,19 @@
 import os
 import unittest
 import logging
+import sys
 
 import dbus
 import dbus.glib
 import dbus.service
-
-from dbus._compat import is_py2
+import dbus.types
+import dbus_test_utils
 
 try:
-    from gi.repository import GObject as gobject
+    from gi.repository import GLib
 except ImportError:
-    raise SystemExit(77)
+    print('1..0 # SKIP cannot import GLib')
+    raise SystemExit(0)
 
 logging.basicConfig()
 logging.getLogger().setLevel(1)
@@ -56,15 +60,10 @@ class TestDBusBindings(unittest.TestCase):
         # using dbus.bus.BusConnection!
         conn = dbus.connection.Connection(
                 os.environ['DBUS_SESSION_BUS_ADDRESS'])
-        kwargs = {}
-        if is_py2:
-            kwargs['utf8_strings'] = True
         unique = conn.call_blocking('org.freedesktop.DBus',
                                     '/org/freedesktop/DBus',
                                     'org.freedesktop.DBus', 'Hello',
-                                    '', (), **kwargs)
-        if is_py2:
-            self.assertTrue(unique.__class__ == dbus.UTF8String, repr(unique))
+                                    '', ())
         self.assertTrue(unique.startswith(':'), unique)
         conn.set_unique_name(unique)
         return conn, unique
@@ -73,6 +72,29 @@ class TestDBusBindings(unittest.TestCase):
         conn, unique = self.get_conn_and_unique()
         ret = conn.call_blocking(NAME, OBJECT, IFACE, 'Echo', 'v', ('V',))
         self.assertEqual(ret, 'V')
+        self.assertEqual(ret.variant_level, 1)
+
+    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix")
+    def testUnixFd(self):
+        with open('/dev/null', 'r') as file_like:
+            for method in 'Echo', 'EchoVariant', 'EchoFd':
+                if method == 'EchoFd':
+                    sig = 'h'
+                else:
+                    sig = 'v'
+
+                conn, unique = self.get_conn_and_unique()
+                ret = conn.call_blocking(NAME, OBJECT, IFACE, method, sig, (dbus.types.UnixFd(file_like),))
+                self.assertEqual(type(ret), dbus.types.UnixFd)
+
+                if method == 'EchoFd':
+                    self.assertEqual(ret.variant_level, 0)
+                elif method == 'EchoVariant':
+                    self.assertEqual(ret.variant_level, 1)
+
+                plain_fd = ret.take()
+                self.assertTrue(os.path.sameopenfile(file_like.fileno(), plain_fd))
+                os.close(plain_fd)
 
     def testCallThroughProxy(self):
         conn, unique = self.get_conn_and_unique()
@@ -83,18 +105,13 @@ class TestDBusBindings(unittest.TestCase):
 
     def testSetUniqueName(self):
         conn, unique = self.get_conn_and_unique()
-        kwargs = {}
-        if is_py2:
-            kwargs['utf8_strings'] = True
         ret = conn.call_blocking(NAME, OBJECT, IFACE,
-                                 'MethodExtraInfoKeywords', '', (),
-                                 **kwargs)
+                                 'MethodExtraInfoKeywords', '', ())
         self.assertEqual(ret, (unique, OBJECT, NAME,
                                 'dbus.lowlevel.MethodCallMessage'))
 
 
 if __name__ == '__main__':
-    gobject.threads_init()
     dbus.glib.init_threads()
 
-    unittest.main()
+    dbus_test_utils.main()
