@@ -6,6 +6,8 @@ run in isolation.
 
 # Copyright (C) 2006 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -28,41 +30,68 @@ run in isolation.
 
 from __future__ import unicode_literals
 
+import logging
+import struct
 import sys
 import os
 import unittest
 
-builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
-pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
-
 import _dbus_bindings
 import dbus
 import dbus.lowlevel as lowlevel
 import dbus.types as types
-from dbus._compat import is_py2, is_py3
-
-if is_py3:
-    def make_long(n):
-        return n
-else:
-    def make_long(n):
-        return long(n)
+import dbus_test_utils
 
 
-# Check that we're using the right versions
-if not dbus.__file__.startswith(pydir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%dbus.__file__)
-if not _dbus_bindings.__file__.startswith(builddir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%_dbus_bindings.__file__)
+logging.basicConfig()
+logging.getLogger().setLevel(1)
+logger = logging.getLogger('test-standalone')
+
+
+if 'DBUS_TEST_UNINSTALLED' in os.environ:
+    builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
+    pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
+    pkg = dbus.__file__
+
+    if not pkg.startswith(pydir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % pkg)
+
+    if not _dbus_bindings.__file__.startswith(builddir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % _dbus_bindings.__file__)
+
+    assert _dbus_bindings.__version__ == os.environ['DBUS_PYTHON_VERSION'], \
+            '_dbus_bindings was compiled as version %s but Automake says '\
+            'we should be version %s' \
+            % (_dbus_bindings.__version__, os.environ['DBUS_PYTHON_VERSION'])
+
 assert (_dbus_bindings._python_version & 0xffff0000
         == sys.hexversion & 0xffff0000), \
         '_dbus_bindings was compiled for Python %x but this is Python %x, '\
         'a different major version'\
         % (_dbus_bindings._python_version, sys.hexversion)
-assert _dbus_bindings.__version__ == os.environ['DBUS_PYTHON_VERSION'], \
-        '_dbus_bindings was compiled as version %s but Automake says '\
-        'we should be version %s' \
-        % (_dbus_bindings.__version__, os.environ['DBUS_PYTHON_VERSION'])
+
+logger.info('dbus found at %r', dbus.__file__)
+logger.info('_dbus_bindings found at %r', _dbus_bindings.__file__)
+
+
+def ignore_cb(*args, **kws):
+    pass
+
+
+def uni(x):
+    """Return a Unicode string consisting of the single Unicode character
+    with the given codepoint (represented as a surrogate pair if this is
+    a "narrow" Python build). This is a generalization of unichr().
+
+    uni(0x0001f639) == u'\\U0001f639' in versions where that syntax is
+    supported.
+    """
+    if x <= 0xFFFF:
+        return chr(x)
+    else:
+        return struct.pack('>I', x).decode('utf_32_be')
 
 class TestTypes(unittest.TestCase):
 
@@ -80,6 +109,9 @@ class TestTypes(unittest.TestCase):
         self.assertEqual(types.Double(0.0), 0.0)
         self.assertEqual(types.Double(0.125, variant_level=2), 0.125)
         self.assertEqual(types.Double(0.125, variant_level=2).variant_level, 2)
+        self.assertEqual(str(types.Double(0.125)), '0.125')
+        self.assertEqual(float(types.Double(0.125)), 0.125)
+        self.assertIs(type(float(types.Double(0.125))), float)
 
     def test_Struct(self):
         x = types.Struct(('',))
@@ -93,9 +125,9 @@ class TestTypes(unittest.TestCase):
         self.assertEqual(types.Byte(b'x', variant_level=2),
                           types.Byte(ord('x')))
         self.assertEqual(types.Byte(1), 1)
-        self.assertEqual(types.Byte(make_long(1)), 1)
         self.assertRaises(Exception, lambda: types.Byte(b'ab'))
         self.assertRaises(TypeError, types.Byte, '\x12xxxxxxxxxxxxx')
+        self.assertEqual(str(types.Byte(b'x')), 'x')
 
         # Byte from a unicode object: what would that even mean?
         self.assertRaises(Exception,
@@ -112,8 +144,6 @@ class TestTypes(unittest.TestCase):
 
     def test_integers(self):
         subclasses = [int]
-        if is_py2:
-            subclasses.append(long)
         subclasses = tuple(subclasses)
         # This is an API guarantee. Note that exactly which of these types
         # are ints and which of them are longs is *not* guaranteed.
@@ -124,6 +154,10 @@ class TestTypes(unittest.TestCase):
             self.assertEqual(cls(0), 0)
             self.assertEqual(cls(23, variant_level=1), 23)
             self.assertEqual(cls(23, variant_level=1).variant_level, 1)
+            self.assertEqual(int(cls(42)), 42)
+            self.assertIs(type(int(cls(42))), int)
+            self.assertEqual(str(cls(42)), '42')
+            self.assertIs(type(str(cls(42))), str)
 
     def test_integer_limits_16(self):
         self.assertEqual(types.Int16(0x7fff), 0x7fff)
@@ -135,27 +169,22 @@ class TestTypes(unittest.TestCase):
 
     def test_integer_limits_32(self):
         self.assertEqual(types.Int32(0x7fffffff), 0x7fffffff)
-        self.assertEqual(types.Int32(make_long(-0x80000000)), 
-                         make_long(-0x80000000))
-        self.assertEqual(types.UInt32(make_long(0xffffffff)), 
-                         make_long(0xffffffff))
-        self.assertRaises(Exception, types.Int32, make_long(0x80000000))
-        self.assertRaises(Exception, types.Int32, make_long(-0x80000001))
-        self.assertRaises(Exception, types.UInt32, make_long(0x100000000))
+        self.assertEqual(types.Int32(-0x80000000), -0x80000000)
+        self.assertEqual(types.UInt32(0xffffffff), 0xffffffff)
+        self.assertRaises(Exception, types.Int32, 0x80000000)
+        self.assertRaises(Exception, types.Int32, -0x80000001)
+        self.assertRaises(Exception, types.UInt32, 0x100000000)
 
     def test_integer_limits_64(self):
-        self.assertEqual(types.Int64(make_long(0x7fffffffffffffff)), 
-                         make_long(0x7fffffffffffffff))
-        self.assertEqual(types.Int64(make_long(-0x8000000000000000)), 
-                         make_long(-0x8000000000000000))
-        self.assertEqual(types.UInt64(make_long(0xffffffffffffffff)), 
-                         make_long(0xffffffffffffffff))
-        self.assertRaises(Exception, types.Int16, 
-                          make_long(0x8000000000000000))
-        self.assertRaises(Exception, types.Int16, 
-                          make_long(-0x8000000000000001))
-        self.assertRaises(Exception, types.UInt16, 
-                          make_long(0x10000000000000000))
+        self.assertEqual(types.Int64(0x7fffffffffffffff),
+                         0x7fffffffffffffff)
+        self.assertEqual(types.Int64(-0x8000000000000000),
+                         -0x8000000000000000)
+        self.assertEqual(types.UInt64(0xffffffffffffffff),
+                         0xffffffffffffffff)
+        self.assertRaises(Exception, types.Int64, 0x8000000000000000)
+        self.assertRaises(Exception, types.Int64, -0x8000000000000001)
+        self.assertRaises(Exception, types.UInt64, 0x10000000000000000)
 
     def test_Signature(self):
         self.assertRaises(Exception, types.Signature, 'a')
@@ -165,6 +194,35 @@ class TestTypes(unittest.TestCase):
                          ('ab', '(xt)', 'a{sv}'))
         self.assertTrue(isinstance(tuple(types.Signature('ab'))[0],
                                    types.Signature))
+        self.assertEqual(str(types.Signature('ab')), 'ab')
+        self.assertIs(type(str(types.Signature('ab'))), str)
+
+    def test_string(self):
+        self.assertEqual(types.String('hello', variant_level=23), 'hello')
+        self.assertEqual(types.String('hello', variant_level=23).variant_level, 23)
+        self.assertTrue(isinstance(types.String('hello'), str))
+        self.assertEqual(str(types.String('hello')), 'hello')
+        self.assertIs(type(str(types.String('hello'))), str)
+
+    def test_object_path(self):
+        self.assertRaises(Exception, types.ObjectPath, 'a')
+        self.assertEqual(types.ObjectPath('/ab', variant_level=23), '/ab')
+        self.assertEqual(types.ObjectPath('/ab', variant_level=23).variant_level, 23)
+        self.assertTrue(isinstance(types.ObjectPath('/ab'), str))
+        self.assertEqual(str(types.ObjectPath('/ab')), '/ab')
+        self.assertIs(type(str(types.ObjectPath('/ab'))), str)
+
+    def test_boolean(self):
+        self.assertEqual(types.Boolean(True, variant_level=23), True)
+        self.assertEqual(types.Boolean(True, variant_level=23).variant_level, 23)
+        self.assertEqual(str(types.Boolean(False)), '0')
+        self.assertEqual(str(types.Boolean(True)), '1')
+        self.assertEqual(str(types.Boolean(47)), '1')
+        self.assertEqual(int(types.Boolean(False)), 0)
+        self.assertEqual(int(types.Boolean(True)), 1)
+        self.assertEqual(int(types.Boolean(47)), 1)
+        self.assertIs(type(int(types.Boolean(False))), int)
+        self.assertIs(type(int(types.Boolean(True))), int)
 
 
 class TestMessageMarshalling(unittest.TestCase):
@@ -329,8 +387,6 @@ class TestMessageMarshalling(unittest.TestCase):
         aeq = self.assertEqual
         from _dbus_bindings import Message
         aeq(Message.guess_signature(7), 'i')
-        if is_py2:
-            aeq(Message.guess_signature(make_long(7)), 'x')
 
     def test_guess_signature_dbus_types(self):
         aeq = self.assertEqual
@@ -367,31 +423,20 @@ class TestMessageMarshalling(unittest.TestCase):
         aeq(byte.__class__, types.Byte)
         aeq(bytes.__class__, types.ByteArray)
         aeq(bytes, b'bytes')
-        if is_py3:
-            aeq(bytes[0].__class__, int)
-        else:
-            aeq(bytes[0].__class__, str)
+        aeq(bytes[0].__class__, int)
         aeq(int32.__class__, types.Int32)
         aeq(uint32.__class__, types.UInt32)
         aeq(string.__class__, types.String)
         aeq(variant.__class__, types.String)
         aeq(variant.variant_level, 1)
 
-        kwargs = {}
-        if is_py2:
-            kwargs['utf8_strings'] = True
-        byte, bytes, int32, uint32, string, variant = s.get_args_list(
-            **kwargs)
+        byte, bytes, int32, uint32, string, variant = s.get_args_list()
         aeq(byte.__class__, types.Byte)
         aeq(bytes.__class__, types.Array)
         aeq(bytes[0].__class__, types.Byte)
         aeq(int32.__class__, types.Int32)
         aeq(uint32.__class__, types.UInt32)
-        if is_py2:
-            aeq(string.__class__, types.UTF8String)
         aeq(string, 'str')
-        if is_py2:
-            aeq(variant.__class__, types.UTF8String)
         aeq(variant.variant_level, 1)
         aeq(variant, 'var')
 
@@ -425,19 +470,10 @@ class TestMessageMarshalling(unittest.TestCase):
 
     def test_utf8(self):
         from _dbus_bindings import SignalMessage
-        if is_py3:
-            def utf8(*xs):
-                return bytes(xs)
-            def uni(x):
-                return chr(x)
-        else:
-            def utf8(*xs):
-                return str('').join(map(chr, xs))
-            def uni(x):
-                return unichr(x)
+
         for bad in [
                 uni(0xD800),
-                utf8(0xed, 0xa0, 0x80),
+                b'\xed\xa0\x80',
                 ]:
             s = SignalMessage('/', 'foo.bar', 'baz')
             try:
@@ -461,43 +497,83 @@ class TestMessageMarshalling(unittest.TestCase):
             s.append(good.encode('utf-8'), signature='s')
         for noncharacter in [
                 uni(0xFDD0),
-                utf8(0xef, 0xb7, 0x90),
+                b'\xef\xb7\x90',
                 uni(0xFDD7),
-                utf8(0xef, 0xb7, 0x97),
+                b'\xef\xb7\x97',
                 uni(0xFDEF),
-                utf8(0xef, 0xb7, 0xaf),
+                b'\xef\xb7\xaf',
                 uni(0xFFFE),
-                utf8(0xef, 0xbf, 0xbe),
+                b'\xef\xbf\xbe',
                 uni(0xFFFF),
-                utf8(0xef, 0xbf, 0xbf),
+                b'\xef\xbf\xbf',
                 uni(0x0001FFFE),
-                utf8(0xf0, 0x9f, 0xbf, 0xbe),
+                b'\xf0\x9f\xbf\xbe',
                 uni(0x0001FFFF),
-                utf8(0xf0, 0x9f, 0xbf, 0xbf),
+                b'\xf0\x9f\xbf\xbf',
                 uni(0x0007FFFE),
-                utf8(0xf1, 0xbf, 0xbf, 0xbe),
+                b'\xf1\xbf\xbf\xbe',
                 uni(0x0007FFFF),
-                utf8(0xf1, 0xbf, 0xbf, 0xbf),
+                b'\xf1\xbf\xbf\xbf',
                 uni(0x0010FFFE),
-                utf8(0xf4, 0x8f, 0xbf, 0xbe),
+                b'\xf4\x8f\xbf\xbe',
                 uni(0x0010FFFF),
-                utf8(0xf4, 0x8f, 0xbf, 0xbf),
+                b'\xf4\x8f\xbf\xbf',
                 ]:
             s = SignalMessage('/', 'foo.bar', 'baz')
             try:
                 s.append(noncharacter, signature='s')
             except UnicodeError:
-                pass  # libdbus < 1.6.10 disallows noncharacters
+                raise AssertionError(
+                    'Appending %r should succeed' % noncharacter)
             else:
                 pass  # libdbus >= 1.6.10 allows noncharacters
 
+    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix")
+    def test_unix_fd(self):
+        plain_fd = os.open('/dev/null', os.O_RDONLY)
+
+        try:
+            with open('/dev/null', 'r') as file_like_object:
+                fd = types.UnixFd(plain_fd)
+                self.assertEqual(fd.variant_level, 0)
+                other = fd.take()
+                os.close(other)
+
+                with self.assertRaises(ValueError):
+                    fd.take()
+
+                fd = types.UnixFd(plain_fd, variant_level=42)
+                self.assertEqual(fd.variant_level, 42)
+
+                fd = types.UnixFd(file_like_object)
+                self.assertEqual(fd.variant_level, 0)
+
+                fd = types.UnixFd(file_like_object, variant_level=42)
+                self.assertEqual(fd.variant_level, 42)
+
+                with self.assertRaises(TypeError):
+                    int(fd)
+
+                with self.assertRaises(TypeError):
+                    types.UnixFd(plain_fd, invalid_kwarg='nope')
+
+                with self.assertRaises(TypeError):
+                    types.UnixFd(plain_fd, variant_level='nope')
+
+                with self.assertRaises(ValueError):
+                    types.UnixFd(plain_fd, variant_level=-1)
+        finally:
+            os.close(plain_fd)
+
+        with self.assertRaises(ValueError):
+            types.UnixFd(-1)
+
 class TestMatching(unittest.TestCase):
     def setUp(self):
         from _dbus_bindings import SignalMessage
         from dbus.connection import SignalMatch
         self._message = SignalMessage('/', 'a.b', 'c')
         class FakeConn(object): pass
-        def ignore_cb(*args, **kws): pass
         self._match = SignalMatch(FakeConn(), None, '/', None, None, 
                                   ignore_cb, arg0='/')
 
@@ -509,9 +585,118 @@ class TestMatching(unittest.TestCase):
         self._message.append('/', signature='o')
         self.assertFalse(self._match.maybe_handle_message(self._message))
 
+class TestArg0Namespace(unittest.TestCase):
+    def setUp(self):
+        self._match = dbus.connection.SignalMatch(object, None, '/', None, None,
+                                  ignore_cb, arg0namespace='org.freedesktop.dbus-python')
+
+    def test_invalid_arg1namespace(self):
+        try:
+            dbus.connection.SignalMatch(object, None, '/', None, None,
+                                  ignore_cb, arg1namespace='org.freedesktop.dbus-python')
+        except TypeError:
+            pass
+        else:
+            raise AssertionError('arg1namespace is not a valid keyword argument')
+
+    def test_namespace_match(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('org.freedesktop.dbus-python', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_namespace_match2(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('org.freedesktop.dbus-python.test', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_namespace_mismatch(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('org.freedesktop.dbus-python-test', signature='s')
+        self.assertFalse(self._match.maybe_handle_message(message))
+
+    def test_namespace_type_mismatch(self):
+        match = dbus.connection.SignalMatch(object, None, '/', None, None,
+                                  ignore_cb, arg0namespace='1')
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append(1, signature='i')
+        self.assertFalse(match.maybe_handle_message(message))
+
+class TestPathMatches(unittest.TestCase):
+    def setUp(self):
+        self._match = dbus.connection.SignalMatch(object, None, '/', None, None,
+                                  ignore_cb, arg0path='/aa/bb/')
+
+    def test_empty_path(self):
+
+        match = dbus.connection.SignalMatch(object, None, '/', None, None,
+                                  ignore_cb, arg0path='')
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/', signature='s')
+        self.assertFalse(match.maybe_handle_message(message))
+
+    def test_empty_value(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('', signature='s')
+        self.assertFalse(self._match.maybe_handle_message(message))
+
+    def test_root(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_aa(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_aa_bb(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/bb/', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_aa_bb_cc(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/bb/cc/', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_aa_bb_cc2(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/bb/cc', signature='s')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+    def test_no_match_aa_b(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/b', signature='s')
+        self.assertFalse(self._match.maybe_handle_message(message))
+
+    def test_no_match_aa(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa', signature='s')
+        self.assertFalse(self._match.maybe_handle_message(message))
+
+    def test_no_match_aa_bb(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/bb', signature='s')
+        self.assertFalse(self._match.maybe_handle_message(message))
+
+    def test_object_path(self):
+        message = _dbus_bindings.SignalMessage('/', 'a.b', 'c')
+        message.append('/aa/bb/cc', signature='o')
+        self.assertTrue(self._match.maybe_handle_message(message))
+
+class TestVersion(unittest.TestCase):
+    if sys.version_info[:2] < (2, 7):
+        def assertGreater(self, first, second):
+            self.assertTrue(first > second)
+
+        def assertLess(self, first, second):
+            self.assertTrue(first < second)
+
+    def test_version(self):
+        self.assertGreater(dbus.version, (0, 41))
+        self.assertLess(dbus.version, (23, 0))
+        self.assertGreater(dbus.__version__, '0')
+        self.assertLess(dbus.__version__, '9')
+
 if __name__ == '__main__':
-    # Python 2.6 doesn't accept a `verbosity` keyword.
-    kwargs = {}
-    if sys.version_info[:2] >= (2, 7):
-        kwargs['verbosity'] = 2
-    unittest.main(**kwargs)
+    dbus_test_utils.main()
